﻿using System.Linq;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;

namespace App37.mt
{
  public class FlexGridLayout
  {
    public int MinimalLayoutSize { get; set; }

    public FlexGridLayoutDefinition PortraitLayout { get; set; }
    public FlexGridLayoutDefinition LandscapeLayout { get; set; }
    public FlexGridLayoutDefinition MinimalLayout { get; set; }

    internal void ClearAssociatedGrid()
    {
      if (this.grid != null)
      {
        this.grid.Loaded -= OnLoaded;
        this.grid.SizeChanged -= OnSizeChanged;
        this.grid = null;
      }
    }
    internal void SetAssociatedGrid(Grid grid)
    {
      this.ClearAssociatedGrid();

      this.grid = grid;

      if (this.grid != null)
      {
        this.grid.Loaded += OnLoaded;
        this.grid.SizeChanged += OnSizeChanged;
      }
    }
    void OnSizeChanged(object sender, SizeChangedEventArgs e)
    {
      if (this.currentShape != null)
      {
        FlexGridLayoutType newShape = this.DetermineGridShape();

        if (newShape != this.currentShape)
        {
          this.currentShape = newShape;
          this.Recreate();
        }
      }
    }
    void OnLoaded(object sender, RoutedEventArgs args)
    {
      this.currentShape = this.DetermineGridShape();
      this.Recreate();
    }
    FlexGridLayoutType DetermineGridShape()
    {
      FlexGridLayoutType shape = FlexGridLayoutType.Landscape;

      if (this.grid != null)
      {
        if (this.grid.ActualWidth < this.grid.ActualHeight)
        {
          shape = FlexGridLayoutType.Portrait;
        }
        if (this.grid.ActualWidth <= this.MinimalLayoutSize)
        {
          shape = FlexGridLayoutType.Minimal;
        }
      }
      return (shape);
    }
    void Recreate()
    {
      if (this.currentShape.HasValue && (this.grid != null))
      {
        this.grid.RowDefinitions.Clear();
        this.grid.ColumnDefinitions.Clear();

        FlexGridLayoutDefinition definition = this.GetDefinitionBasedOnCurrentShape();

        foreach (var rowDef in definition.RowDefinitions)
        {
          this.grid.RowDefinitions.Add(rowDef);
        }
        foreach (var colDef in definition.ColumnDefinitions)
        {
          this.grid.ColumnDefinitions.Add(colDef);
        }
        foreach (var change in definition.Changes.Where(ch => ch.Validate()))
        {
          FrameworkElement element = this.grid.FindName(change.ElementName) as FrameworkElement;

          if (element != null)
          {
            if (change.Type == FlexGridLayoutChangeType.Toggle)
            {
              element.Visibility = element.Visibility == Visibility.Visible ?
                Visibility.Collapsed : Visibility.Visible;
            }
            else
            {
              FlexGridLayoutPositionChange posChange = change as FlexGridLayoutPositionChange;

              if (posChange != null)
              {
                Grid.SetColumn(element, posChange.Column);
                Grid.SetRow(element, posChange.Row);
              }
            }
          }
        }
      }
    }
    FlexGridLayoutDefinition GetDefinitionBasedOnCurrentShape()
    {
      FlexGridLayoutDefinition definition = null;

      switch (this.currentShape.Value)
      {
        case FlexGridLayoutType.Landscape:
          definition = this.LandscapeLayout;
          break;
        case FlexGridLayoutType.Portrait:
          definition = this.PortraitLayout;
          break;
        case FlexGridLayoutType.Minimal:
          definition = this.MinimalLayout;
          break;
        default:
          break;
      }
      if (definition == null)
      {
        definition = this.LandscapeLayout;
      }
      return (definition);
    }
    FlexGridLayoutType? currentShape;
    Grid grid;
  }
}
